@@ -1,23 +1,29 @@
 use core::ops::FnMut;
 
-use nom::branch::alt;
-use nom::bytes::complete::{escaped_transform, is_not, tag};
-use nom::character::complete::{char, line_ending};
-use nom::combinator::{flat_map, map_res, value};
+use nom::bytes::complete::{escaped, is_not};
+use nom::character::complete::{char, line_ending, one_of};
+use nom::combinator::{flat_map, map_res};
 use nom::error::context;
 use nom::multi::many0;
 use nom::sequence::terminated;
 use nom::IResult;
 use nom::Parser;
-use std::str::FromStr;
 
 use crate::error::{FullError, StompParseError};
 use crate::model::headers::parser::*;
 use crate::model::headers::*;
 
-#[allow(type_alias_bounds)]
-type HeaderParser<'a, E: 'static + FullError<&'a [u8], StompParseError>> =
-    dyn FnMut(&'a [u8]) -> IResult<&'a [u8], Header, E> + 'a;
+trait HeaderParser<'a, E: FullError<&'a [u8], StompParseError>>:
+    FnMut(&'a [u8]) -> IResult<&'a [u8], Header, E> + 'a
+{
+}
+
+impl<'a, E, T> HeaderParser<'a, E> for T
+where
+    E: FullError<&'a [u8], StompParseError>,
+    T: FnMut(&'a [u8]) -> IResult<&'a [u8], Header, E> + 'a,
+{
+}
 
 /// Creates an new HeadersParser accepting the specified required and optional Headers,
 /// and optionally arbitrary other headers as "custom" headers.
@@ -41,7 +47,7 @@ fn init_headers_parser<'a, E>(
     required: Vec<HeaderType>,
     optional: Vec<HeaderType>,
     allows_custom: bool,
-) -> Box<dyn Fn(String) -> Box<HeaderParser<'a, E>> + 'a>
+) -> Box<dyn Fn(&'a str) -> Box<dyn HeaderParser<'a, E>> + 'a>
 where
     E: 'a + FullError<&'a [u8], StompParseError>,
 {
@@ -50,61 +56,76 @@ where
 
     // The part that deals with any other headers encountered
     //let custom_header_parser_provider = custom_header_parser_provider_factory(allows_custom);
-    Box::new(move |name: String| {
-        HeaderType::from_str(&name) // Determine the type
-            .and_then(&*known_headers) // Then see if it is a known header, and return the appropriate parser
+    Box::new(move |name: &'a str| {
+        // Determine the type
+        known_headers(name) // Then see if it is a known header, and return the appropriate parser
             .unwrap_or_else(|_| disallowed_header_parser(name))
     })
 }
 
+fn find_header<'a, 'b, E>(
+    name: &'a str,
+    required: &'b Vec<HeaderType>,
+    optional: &'b Vec<HeaderType>,
+    allows_custom: bool,
+) -> Result<Box<dyn HeaderParser<'a, E> + 'a>, StompParseError>
+where
+    'a: 'b,
+    E: 'a + FullError<&'a [u8], StompParseError>,
+{
+    required
+        .iter()
+        .find(|header_type| header_type.matches(name))
+        .or_else(|| {
+            optional
+                .iter()
+                .find(|header_type| header_type.matches(name))
+        })
+        .map(|header_type| {
+            Ok(known_header_parser::<'a, E>(find_header_parser(
+                *header_type,
+            )))
+        })
+        .unwrap_or_else(|| {
+            if allows_custom {
+                Ok(known_header_parser::<'a, E>(Box::new(
+                    move |value: &str| {
+                        Ok(Header::Custom(CustomValue::new(
+                            unsafe { std::mem::transmute::<&'a str, &'static str>(name) },
+                            unsafe { std::mem::transmute::<&'_ str, &'static str>(value) },
+                        )))
+                    },
+                )))
+            } else {
+                Err(StompParseError::new(format!("Unknown header: {}", name)))
+            }
+        })
+}
+
 fn init_known_header_parser<'a, E>(
     required: Vec<HeaderType>,
     optional: Vec<HeaderType>,
     allows_custom: bool,
-) -> Box<dyn Fn(HeaderType) -> Result<Box<HeaderParser<'a, E>>, StompParseError>>
+) -> impl Fn(&'a str) -> Result<Box<dyn HeaderParser<'a, E>>, StompParseError> + 'a
 where
     E: 'a + FullError<&'a [u8], StompParseError>,
 {
-    Box::new(move |header_type| {
-        // Check if its one of the known accepted headers
-        if required.contains(&header_type) || optional.contains(&header_type) {
-            Ok(known_header_parser::<'a, E>(find_header_parser(
-                header_type,
-            )))
-        } else if allows_custom {
-            // Otherwise, if custom is allowed, save it as a custom header
-            Ok(known_header_parser::<'a, E>(find_header_parser(
-                // This converts 'known-but-not-accepted' headers to custom ones
-                HeaderType::Custom(header_type.to_string()),
-            )))
-        } else {
-            Err(StompParseError::new(""))
-        }
-    })
+    move |name: &'a str| find_header(name, &required, &optional, allows_custom)
 }
 
 fn header_section<'a, E: FullError<&'a [u8], StompParseError>>(
     input: &'a [u8],
-) -> IResult<&'a [u8], Vec<u8>, E> {
-    escaped_transform(
-        is_not("\\:\n\r"),
-        '\\',
-        alt((
-            value(b"\n" as &[u8], tag("n")),
-            value(b"\r" as &[u8], tag("r")),
-            value(b":" as &[u8], tag("c")),
-            value(b"\\" as &[u8], tag("\\")),
-        )),
-    )(input)
+) -> IResult<&'a [u8], &'a [u8], E> {
+    escaped(is_not("\\:\n\r"), '\\', one_of("rnc\\"))(input)
 }
 
-fn into_string(input: Vec<u8>) -> Result<String, StompParseError> {
-    String::from_utf8(input).map_err(|_| StompParseError::new("bytes are not utf8"))
+fn into_string<'a>(input: &'a [u8]) -> Result<&'a str, StompParseError> {
+    std::str::from_utf8(input).map_err(|_| StompParseError::new("bytes are not utf8"))
 }
 
 fn header_name<'a, E: FullError<&'a [u8], StompParseError>>(
     input: &'a [u8],
-) -> IResult<&'a [u8], String, E> {
+) -> IResult<&'a [u8], &'a str, E> {
     context(
         "header name",
         map_res(terminated(header_section, char(':')), into_string),
@@ -113,7 +134,7 @@ fn header_name<'a, E: FullError<&'a [u8], StompParseError>>(
 
 fn header_value<'a, E: FullError<&'a [u8], StompParseError>>(
     input: &'a [u8],
-) -> IResult<&'a [u8], String, E> {
+) -> IResult<&'a [u8], &'a str, E> {
     context(
         "header value",
         map_res(terminated(header_section, line_ending), into_string),
@@ -121,8 +142,8 @@ fn header_value<'a, E: FullError<&'a [u8], StompParseError>>(
 }
 
 fn disallowed_header_parser<'a, E: 'a + FullError<&'a [u8], StompParseError>>(
-    name: String,
-) -> Box<HeaderParser<'a, E>> {
+    name: &'a str,
+) -> Box<dyn HeaderParser<'a, E>> {
     Box::new(map_res(header_value, move |_| {
         Err(StompParseError::new(format!(
             "Unexpected header '{}' encountered",
@@ -132,13 +153,14 @@ fn disallowed_header_parser<'a, E: 'a + FullError<&'a [u8], StompParseError>>(
 }
 
 fn known_header_parser<'a, E: 'a + FullError<&'a [u8], StompParseError>>(
-    parser: Box<dyn Fn(String) -> Result<Header, StompParseError>>,
-) -> Box<HeaderParser<'a, E>> {
+    parser: Box<dyn Fn(&str) -> Result<Header, StompParseError> + 'a>,
+) -> Box<dyn HeaderParser<'a, E>> {
     Box::new(map_res(header_value, parser))
 }
 
 #[cfg(test)]
 mod tests {
+    use either::Either;
     use nom::error::VerboseError;
 
     use super::headers_parser;
@@ -201,27 +223,62 @@ mod tests {
         input: &'static str,
         expected_key: &'static str,
         expected_value: &'static str,
+        expected_decoded_key: Option<&'static str>,
+        expected_decoded_value: Option<&'static str>,
     ) {
         let result = headers::<VerboseError<&'static [u8]>>(input.as_bytes())
             .unwrap()
             .1;
 
         if let Header::Custom(value) = &result[0] {
-            assert_eq!(*expected_key, *value.header_name());
-            assert_eq!(*expected_value, *value.value());
+            assert_eq!(expected_key, value.header_name());
+            check_raw_and_decoded(
+                expected_key,
+                value.header_name(),
+                expected_decoded_key,
+                value.decoded_name(),
+            );
+            check_raw_and_decoded(
+                expected_value,
+                value.value(),
+                expected_decoded_value,
+                value.decoded_value(),
+            );
         } else {
             panic!("Expected custom header");
         }
     }
 
+    fn check_raw_and_decoded(
+        expected_value: &str,
+        actual_value: &str,
+        expected_decoded_value: Option<&str>,
+        actual_decoded: Result<Either<&str, String>, StompParseError>,
+    ) {
+        assert_eq!(expected_value, actual_value);
+        if let Some(expected_decoded_value) = expected_decoded_value {
+            match actual_decoded {
+                Ok(Either::Left(val)) => {
+                    assert_eq!(expected_decoded_value, val);
+                }
+                Ok(Either::Right(val)) => {
+                    assert_eq!(expected_decoded_value, &val);
+                }
+                Err(_) => {
+                    panic!("Decode failed!")
+                }
+            }
+        }
+    }
+
     #[test]
     fn header_line_terminated_by_rn() {
-        assert_custom_header("abc:def\r\n\n", "abc", "def");
+        assert_custom_header("abc:def\r\n\n", "abc", "def", Some("abc"), Some("def"));
     }
 
     #[test]
     fn header_line_terminated_by_n() {
-        assert_custom_header("abc:def\n\n", "abc", "def");
+        assert_custom_header("abc:def\n\n", "abc", "def", Some("abc"), None);
     }
 
     #[test]
@@ -245,25 +302,44 @@ mod tests {
         assert!(result.is_err());
     }
 
+    use nom::bytes::complete::{escaped, is_not};
+    use nom::character::complete::one_of;
+
+    fn esc(input: &[u8]) -> IResult<&[u8], &[u8]> {
+        escaped(is_not("\\:\n\r"), '\\', one_of("rn:\\"))(input)
+    }
+
+    #[test]
+    fn test_escaped() {
+        let (_, matched) = esc(b"a\\rbc:def\n\n" as &[u8]).expect("Should be fine");
+
+        assert_eq!(b"a\\rbc", matched)
+    }
+
     #[test]
     fn header_accepts_escaped_cr() {
-        assert_custom_header("a\\rbc:def\n\n", "a\rbc", "def");
+        assert_custom_header("a\\rbc:def\n\n", "a\\rbc", "def", Some("a\rbc"), None);
     }
 
     #[test]
     fn header_line_accepts_escaped_nl() {
-        assert_custom_header("abc:d\\nef\n\n", "abc", "d\nef");
-        assert_custom_header("abc:d\\nef\n\n", "abc", "d\nef");
+        assert_custom_header(
+            "abc:d\\nef\n\n",
+            "abc",
+            "d\\nef",
+            Some("abc"),
+            Some("d\nef"),
+        );
     }
 
     #[test]
     fn header_line_accepts_escaped_colon() {
-        assert_custom_header("abc:d\\cef\n\n", "abc", "d:ef");
+        assert_custom_header("abc:d\\cef\n\n", "abc", "d\\cef", None, Some("d:ef"));
     }
 
     #[test]
     fn header_accepts_fwd_slash() {
-        assert_custom_header("abc:d\\\\ef\n\n", "abc", "d\\ef");
+        assert_custom_header("abc:d\\\\ef\n\n", "abc", "d\\\\ef", None, Some("d\\ef"));
     }
 
     #[test]
@@ -275,7 +351,13 @@ mod tests {
 
     #[test]
     fn header_works_for_custom() {
-        assert_custom_header("a\\rbc:d\\\\ef\n\n", "a\rbc", "d\\ef");
+        assert_custom_header(
+            "a\\rbc:d\\\\ef\n\n",
+            "a\\rbc",
+            "d\\\\ef",
+            Some("a\rbc"),
+            Some("d\\ef"),
+        );
     }
 
     #[test]
@@ -285,7 +367,7 @@ mod tests {
             .1;
 
         if let Header::Host(value) = header {
-            assert_eq!("d\nef", value.value());
+            assert_eq!("d\\nef", value.value());
         } else {
             panic!("Expected host header");
         }
@@ -314,7 +396,13 @@ mod tests {
     #[test]
     fn header_is_case_sensitive() {
         //heart-beat not recognised
-        assert_custom_header("heArt-beat:10,20\n\n", "heArt-beat", "10,20");
+        assert_custom_header(
+            "heArt-beat:10,20\n\n",
+            "heArt-beat",
+            "10,20",
+            Some("heArt-beat"),
+            None,
+        );
     }
 
     #[test]
@@ -360,7 +448,7 @@ mod tests {
             headers[0]
         );
         assert_eq!(
-            Header::Custom(CustomValue::new("abc".to_string(), "d\nef".to_string())),
+            Header::Custom(CustomValue::new("abc", "d\\nef")),
             headers[1]
         );
     }
